@@ -9,7 +9,7 @@ setClass("enve.RecPlot2",
    id.counts='numeric',		##<< Counts per ID bin.
    id.breaks='numeric',		##<< Breaks of identity bins.
    pos.breaks='numeric',	##<< Breaks of position bins.
-   seq.breaks='numeric',
+   seq.breaks='numeric',	##<< Breaks of input sequences.
    peaks='list',                ##<< Peaks identified in the recplot.
    ### Limits of the subject sequences after concatenation.
    seq.names='character',	##<< Names of the subject sequences.
@@ -87,6 +87,9 @@ plot.enve.RecPlot2 <- function
       peaks.col='darkred',
       ### If not NA, it attempts to represent peaks in the population histogram
       ### in the specified color. Set to NA to avoid peak-finding.
+      use.peaks,
+      ### A list of `enve.RecPlot2.Peak` objects, as returned by
+      ### `enve.recplot2.findPeaks`. If passed, `peaks.opts` is ignored.
       id.lim=range(x$id.breaks),
       ### Limits of identities to represent.
       pos.lim=range(x$pos.breaks),
@@ -102,15 +105,17 @@ plot.enve.RecPlot2 <- function
       ### the number of the panel as index (see `layout`).
       pos.splines=0,
       ### Smoothing parameter for the splines in the position histogram. Zero
-      ### (0) for no splines.
+      ### (0) for no splines. Use NULL to automatically detect by leave-one-out
+      ### cross-validation.
       id.splines=1/2,
       ### Smoothing parameter for the splines in the identity histogram. Zero
-      ### (0) for no splines. If non-zero, requires the stats package.
-      in.lwd=ifelse(pos.splines>0, 1/2, 2),
+      ### (0) for no splines. Use NULL to automatically detect by leave-one-out
+      ### cross-validation.
+      in.lwd=ifelse(is.null(pos.splines) || pos.splines>0, 1/2, 2),
       ### Line width for the sequencing depth of in-group matches.
-      out.lwd=ifelse(pos.splines>0, 1/2, 2),
+      out.lwd=ifelse(is.null(pos.splines) || pos.splines>0, 1/2, 2),
       ### Line width for the sequencing depth of out-group matches.
-      id.lwd=ifelse(id.splines>0, 1/2, 2),
+      id.lwd=ifelse(is.null(id.splines) || id.splines>0, 1/2, 2),
       ### Line width for the identity histogram.
       in.col='darkblue',
       ### Color associated to in-group matches.
@@ -197,7 +202,7 @@ plot.enve.RecPlot2 <- function
       pos.f <- rep(seqdepth.in,each=2)
       lines(pos.x, rep(seqdepth.out,each=2), lwd=out.lwd, col=out.col);
       lines(pos.x, pos.f, lwd=in.lwd, col=in.col);
-      if(pos.splines > 0){
+      if(is.null(pos.splines) || pos.splines > 0){
 	 pos.spline <- smooth.spline(pos.x[pos.f>0], log(pos.f[pos.f>0]),
 	    spar=pos.splines)
 	 lines(pos.spline$x, exp(pos.spline$y), lwd=2, col=in.col)
@@ -234,7 +239,7 @@ plot.enve.RecPlot2 <- function
 	 id.f <- rep(id.counts,each=2)
 	 id.x <- rep(id.breaks,each=2)[-c(1,2*length(id.breaks))]
 	 lines(id.f, id.x, lwd=id.lwd, col=id.col);
-	 if(id.splines > 0){
+	 if(is.null(id.splines) || id.splines > 0){
 	    id.spline <- smooth.spline(id.x[id.f>0], log(id.f[id.f>0]),
 	       spar=id.splines)
 	    lines(exp(id.spline$y), id.spline$x, lwd=2, col=id.col)
@@ -271,8 +276,12 @@ plot.enve.RecPlot2 <- function
       polygon(c(0,rep(h.in$counts,each=2),0,0,rep(sum(pos.counts.in==0),2),0),
 	 y.tmp.in, border=NA, col=in.col)
       if(!is.na(peaks.col)){
-	 o	<- peaks.opts; o$x = x;
-	 peaks	<- do.call(enve.recplot2.findPeaks, o);
+	 o <- peaks.opts; o$x = x;
+         if(missing(use.peaks)){
+           peaks <- do.call(enve.recplot2.findPeaks, o)
+         }else{
+           peaks <- use.peaks
+         }
 	 h.mids <- (10^h.breaks[-1] + 10^h.breaks[-length(h.breaks)])/2
 	 if(!is.null(peaks) & length(peaks)>0){
 	    pf <- h.mids*0;
@@ -286,14 +295,17 @@ plot.enve.RecPlot2 <- function
             dpt <- signif(as.numeric(lapply(peaks, function(x) x$seq.depth)),2)
             frx <- signif(100*as.numeric(
                   lapply(peaks,
-                    function(x) ifelse(length(x$values)==0, x$n.hat, length(x$values))/x$n.total)), 2)
+                    function(x) ifelse(length(x$values)==0, x$n.hat,
+                      length(x$values))/x$n.total)), 2)
             if(peaks[[1]]$err.res < 0){
               err <- paste(', LL:', signif(peaks[[1]]$err.res, 3))
             }else{
-              err <- paste(', err:', signif(as.numeric(lapply(peaks, function(x) x$err.res)), 2))
+              err <- paste(', err:',
+                    signif(as.numeric(lapply(peaks, function(x) x$err.res)), 2))
             }
 	    legend('topright', bty='n', cex=1/2,
-                  legend=paste(letters[1:length(peaks)],'. ', dpt,'X (', frx, '%', err, ')', sep=''))
+                  legend=paste(letters[1:length(peaks)],'. ',
+                    dpt,'X (', frx, '%', err, ')', sep=''))
 	 }
       }
    }
@@ -383,7 +395,7 @@ enve.recplot2 <- function(
       if(pos.breaks>0){
          pos.breaks <- seq(min(lim[,2]), max(lim[,3]), length.out=pos.breaks+1);
       }else{
-         pos.breaks <- c(lim[,2], tail(lim[,3], n=1))
+         pos.breaks <- c(lim[1,2], lim[,3])
       }
    }
    if(length(id.breaks)==1){
@@ -445,7 +457,8 @@ enve.recplot2.findPeaks <- function(
     ### "em" (Expectation-Maximization),
     ### "mower" (Custom distribution-mowing method).
     ...
-    ### Any additional parameters supported by `enve.recplot2.findPeaks.<method>`.
+    ### Any additional parameters supported by
+    ### `enve.recplot2.findPeaks.<method>`.
     ){
   if(method == "emauto"){
     peaks <- enve.recplot2.findPeaks.emauto(x, ...)
@@ -461,9 +474,9 @@ enve.recplot2.findPeaks <- function(
 }
 
 enve.recplot2.findPeaks.emauto <- function(
-  ### Identifies peaks in the population histogram using a Gaussian Mixture Model
-  ### Expectation Maximization (GMM-EM) method with number of components automatically
-  ### detected.
+  ### Identifies peaks in the population histogram using a Gaussian Mixture
+  ### Model Expectation Maximization (GMM-EM) method with number of components
+  ### automatically detected.
     x,
     ### An `enve.RecPlot2` object.
     components=seq(1,10),
@@ -489,26 +502,30 @@ enve.recplot2.findPeaks.emauto <- function(
     stop('Invalid criterion ', criterion)
   }
   for(comp in components){
-    best <- enve.recplot2.findPeaks.__emauto_one(x, comp, do_crit, best, verbose, ...)
+    best <- enve.recplot2.findPeaks.__emauto_one(x, comp, do_crit, best,
+          verbose, ...)
   }
 
-  seqdepths.r <- signif(log(sapply(best[['peaks']], function(x) x$seq.depth)), merge.tol)
+  seqdepths.r <- signif(log(sapply(best[['peaks']],
+        function(x) x$seq.depth)), merge.tol)
   distinct <- length(unique(seqdepths.r))
   if(distinct < length(best[['peaks']])){
     if(verbose) cat('Attempting merge to', distinct, 'components\n')
     init <- apply(sapply(best[['peaks']],
           function(x) c(x$param.hat, alpha=x$n.hat/x$n.total)), 1, as.numeric)
     init <- init[!duplicated(seqdepths.r),]
-    init <- list(mu=init[,'mean'], sd=init[,'sd'], alpha=init[,'alpha']/sum(init[,'alpha']))
-    best <- enve.recplot2.findPeaks.__emauto_one(x, distinct, do_crit, best, verbose, ...)
+    init <- list(mu=init[,'mean'], sd=init[,'sd'],
+          alpha=init[,'alpha']/sum(init[,'alpha']))
+    best <- enve.recplot2.findPeaks.__emauto_one(x, distinct, do_crit, best,
+          verbose, ...)
   }
   return(best[['peaks']])
   ### Returns a list of `enve.RecPlot2.Peak` objects.
 }
 
 enve.recplot2.findPeaks.em <- function(
-  ### Identifies peaks in the population histogram using a Gaussian Mixture Model
-  ### Expectation Maximization (GMM-EM) method.
+  ### Identifies peaks in the population histogram using a Gaussian Mixture
+  ### Model Expectation Maximization (GMM-EM) method.
     x,
     ### An `enve.RecPlot2` object.
     max.iter=1000,
@@ -525,11 +542,13 @@ enve.recplot2.findPeaks.em <- function(
     verbose=FALSE,
     ### Display (mostly debugging) information.
     init,
-    ### Initialization parameters. By default, these are derived from k-means clustering.
-    ### A named list with vectors for 'mu', 'sd', and 'alpha', each of length `components`.
+    ### Initialization parameters. By default, these are derived from k-means
+    ### clustering. A named list with vectors for 'mu', 'sd', and 'alpha', each
+    ### of length `components`.
     log=TRUE
-    ### Logical value indicating if the estimations should be performed in natural
-    ### logarithm units. Do not change unless you know what you're doing.
+    ### Logical value indicating if the estimations should be performed in
+    ### natural logarithm units. Do not change unless you know what you're
+    ### doing.
   ){
   
   # Essential vars
@@ -627,8 +646,9 @@ enve.recplot2.findPeaks.mower <- function(
       verbose=FALSE,
       ### Display (mostly debugging) information.
       log=TRUE
-      ### Logical value indicating if the estimations should be performed in natural
-      ### logarithm units. Do not change unless you know what you're doing.
+      ### Logical value indicating if the estimations should be performed in
+      ### natural logarithm units. Do not change unless you know what you're
+      ### doing.
    ){
    
    # Essential vars
@@ -705,7 +725,8 @@ enve.recplot2.corePeak <- function
    corePeak <- maxPeak
    for(p in x){
      p.len <- ifelse(length(p$values)==0, p$n.hat, length(p$values))
-     corePeak.len <- ifelse(length(corePeak$values)==0, corePeak$n.hat, length(corePeak$values))
+     corePeak.len <- ifelse(
+           length(corePeak$values)==0, corePeak$n.hat, length(corePeak$values))
      sz.d <- log(p.len/corePeak.len)
      if(is.nan(sz.d) || sz.d < 0) next
      sq.d.a <- as.numeric(tail(p$param.hat, n=1))
@@ -743,41 +764,47 @@ enve.recplot2.extractWindows <- function
       (rp,
       ### Recruitment plot, a enve.RecPlot2 object.
       peak,
-      ### Peak, a enve.RecPlot2.Peak object. If list, it is assumed to be a list
-      ### of enve.RecPlot2.Peak objects, in which case the core peak is used
-      ### (see enve.recplot2.corePeak).
+      ### Peak, an `enve.RecPlot2.Peak` object. If list, it is assumed to be a
+      ### list of enve.RecPlot2.Peak objects, in which case the core peak is
+      ### used (see `enve.recplot2.corePeak`).
       lower.tail=TRUE,
       ### If FALSE, it returns windows significantly above the peak in
       ### sequencing depth.
       significance=0.05,
       ### Significance threshold (alpha) to select windows.
       seq.names=FALSE
-      ### Returns subject sequence names instead of a vector of Booleans. It
-      ### assumes that the recruitment plot was generated with pos.breaks=0.
+      ### Returns subject sequence names instead of a vector of Booleans. If
+      ### the recruitment plot was generated with pos.breaks=0 it returns a
+      ### vector of characters (the sequence identifiers), otherwise it returns
+      ### a data.frame with a name column and two columns of coordinates.
       ){
    # Determine the threshold
    if(is.list(peak)) peak <- enve.recplot2.corePeak(peak)
    par <- peak$param.hat
    par[["p"]] <- ifelse(lower.tail, significance, 1-significance)
    thr <- do.call(ifelse(length(par)==4, qsn, qnorm), par)
+   if(peak$log) thr <- exp(thr)
    
-   # Estimate sequencing depths per window
-   pos.cnts.in <- rp$pos.counts.in
-   pos.breaks  <- rp$pos.breaks
-   pos.binsize <- (pos.breaks[-1] - pos.breaks[-length(pos.breaks)])
-   seqdepth.in <- pos.cnts.in/pos.binsize
-
    # Select windows past the threshold
+   seqdepth.in <- enve.recplot2.seqdepth(rp)
    if(lower.tail){
       sel <- seqdepth.in < thr
    }else{
       sel <- seqdepth.in > thr
    }
+   
+   # seq.names=FALSE
    if(!seq.names) return(sel)
-   if(length(seqdepth.in) != length(rp$seq.names))
-      stop(paste("Requesting subject sequence names, but the recruitment plot",
-         "was not generated with pos.breaks=0."))
-   return(rp$seq.names[sel])
+   # seq.names=TRUE and pos.breaks=0
+   if(length(rp$pos.breaks)==length(rp$seq.breaks) &&
+         rp$pos.breaks==rp$seq.breaks)
+           return(rp$seq.names[sel])
+   # seq.names=TRUE and pos.breaks!=0
+   return(enve.recplot2.coordinates(rp,sel))
+   ### Returns a vector of logicals if `seq.names=FALSE`. If `seq.names=TRUE`,
+   ### it returns a vector of characters if the object was built with
+   ### `pos.breaks=0` or a data.frame with four columns otherwise: name.from,
+   ### name.to, pos.from, and pos.to (see `enve.recplot2.coordinates`).
 }
 
 enve.recplot2.compareIdentities <- function
@@ -793,6 +820,10 @@ enve.recplot2.compareIdentities <- function
     ### "bhattacharyya" (Bhattacharyya, 1943, Bull. Calcutta Math. Soc. 35),
     ### "kl" or "kullback-leibler" (Kullback & Leibler, 1951,
     ### doi:10.1214/aoms/1177729694), or "euclidean".
+    smooth.par=NULL,
+    ### Smoothing parameter for cubic spline smoothing. Use 0 for no smoothing.
+    ### Use NULL to automatically determine this value using leave-one-out
+    ### cross-validation (see `smooth.spline` parameter `spar`).
     pseudocounts=0,
     ### Smoothing parameter for Laplace smoothing. Use 0 for no smoothing, or
     ### 1 for add-one smoothing.
@@ -813,9 +844,19 @@ enve.recplot2.compareIdentities <- function
   if(dev > max.deviation)
     stop("'x' and 'y' must have similar `id.breaks`; exceeding max.deviation: ",
           dev)
+  x.cnt <- x$id.counts
+  y.cnt <- y$id.counts
+  if(is.null(smooth.par) || smooth.par > 0){
+    x.mids <- (x$id.breaks[-1] + x$id.breaks[-length(x$id.breaks)])/2
+    y.mids <- (y$id.breaks[-1] + y$id.breaks[-length(y$id.breaks)])/2
+    p.spline <- smooth.spline(x.mids, x.cnt, spar=smooth.par)
+    q.spline <- smooth.spline(y.mids, y.cnt, spar=smooth.par)
+    x.cnt <- pmax(p.spline$y, 0)
+    y.cnt <- pmax(q.spline$y, 0)
+  }
   a <- as.numeric(pseudocounts)
-  p <- (x$id.counts + a) / sum(x$id.counts + a)
-  q <- (y$id.counts + a) / sum(y$id.counts + a)
+  p <- (x.cnt + a) / sum(x.cnt + a)
+  q <- (y.cnt + a) / sum(y.cnt + a)
   d <- NA
   if(i.meth %in% c(1L, 2L)){
     d <- sqrt(sum((sqrt(p) - sqrt(q))**2))/sqrt(2)
@@ -831,6 +872,96 @@ enve.recplot2.compareIdentities <- function
   return(d)
 }
 
+enve.recplot2.coordinates <- function
+  ### Returns the sequence name and coordinates of the requested position bins.
+    (x,
+    ### `enve.RecPlot2` object.
+    bins
+    ### Vector of selected bins to return. It can be a vector of logical values
+    ### with the same length as `x$pos.breaks`-1 or a vector of integers. If
+    ### missing, returns the coordinates of all windows.
+    ){
+  if(!inherits(x, "enve.RecPlot2"))
+    stop("'x' must inherit from class `enve.RecPlot2`")
+  if(missing(bins)) bins <- rep(TRUE, length(x$pos.breaks)-1)
+  if(!is.vector(bins)) stop("'bins' must be a vector")
+  if(inherits(bins, "logical")) bins <- which(bins)
+
+  y <- data.frame(stringsAsFactors=FALSE, row.names=bins)
+  
+  for(i in 1:length(bins)){
+    j <- bins[i]
+    # Concatenated coordinates
+    cc <- x$pos.breaks[c(j, j+1)]
+    # Find the corresponding `seq.breaks`
+    sb.from <- which(
+          cc[1] >=x$seq.breaks[-length(x$seq.breaks)] &
+          cc[1] < x$seq.breaks[-1])
+    sb.to   <- which(
+          cc[2] > x$seq.breaks[-length(x$seq.breaks)] &
+          cc[2] <=x$seq.breaks[-1])
+    # Translate coordinates
+    if(length(sb.from)==1 & length(sb.to)==1){
+      y[i, 'name.from'] <- x$seq.names[sb.from]
+      y[i, 'pos.from'] <- floor(x$seq.breaks[sb.from] + cc[1] - 1)
+      y[i, 'name.to']   <- x$seq.names[sb.to]
+      y[i, 'pos.to']   <- ceiling(x$seq.breaks[sb.to] + cc[2] - 1)
+    }
+  }
+
+  return(y)
+  ### Returns a data.frame with four columns: name.from (character), pos.from
+  ### (numeric) name.to (character), and pos.to (numeric). The first two
+  ### correspond to sequence and position of the start point of the bin, the
+  ### last two correspond to the sequence and position of the end point of the
+  ### bin.
+}
+
+enve.recplot2.seqdepth <- function
+  ### Calculate the sequencing depth of the given window(s)
+    (x,
+    ### `enve.RecPlot2` object.
+    sel,
+    ### Window(s) for which the sequencing depth is to be calculated. If not
+    ### passed, it returns the sequencing depth of all windows
+    low.identity=FALSE
+    ### A logical indicating if the sequencing depth is to be estimated only
+    ### with low-identity matches. By default, only high-identity matches are
+    ### used.
+    ){
+  if(!inherits(x, "enve.RecPlot2"))
+    stop("'x' must inherit from class `enve.RecPlot2`")
+  pos.cnts.in <- x$pos.counts.in
+  pos.breaks  <- x$pos.breaks
+  pos.binsize <- (pos.breaks[-1] - pos.breaks[-length(pos.breaks)])
+  seqdepth.in <- pos.cnts.in/pos.binsize
+  if(missing(sel)) return(seqdepth.in)
+  return(seqdepth.in[sel])
+  ### Returns a numeric vector of sequencing depths (in bp/bp). 
+}
+
+enve.recplot2.ANIr <- function
+  ### Estimate the Average Nucleotide Identity from reads (ANIr) from a
+  ### recruitment plot
+    (x,
+    ### `enve.RecPlot2` object.
+    range=c(0,Inf)
+    ### Range of identities to be considered. By default, the full range
+    ### is used (note that the upper boundary is `Inf` and not 100 because
+    ### recruitment plots can also be built with bit-scores). To use only
+    ### intra-population matches (with identities), use c(95,100). To use only
+    ### inter-population values, use c(0,95).
+    ){
+  if(!inherits(x, "enve.RecPlot2"))
+    stop("'x' must inherit from class `enve.RecPlot2`")
+  id.b <- x$id.breaks
+  id <- (id.b[-1]+id.b[-length(id.b)])/2
+  cnt <- x$id.counts
+  cnt[id < range[1]] <- 0
+  cnt[id > range[2]] <- 0
+  return(sum(id*cnt/sum(cnt)))
+}
+
 #==============> Define internal functions
 enve.recplot2.__counts <- function
    ### Internal ancilliary function (see `enve.recplot2`).
@@ -873,7 +1004,8 @@ enve.recplot2.findPeaks.__em_e <- function
   components <- length(theta[['mu']])
   product <- do.call(cbind,
         lapply(1:components,
-          function(i) dnorm(x, theta[['mu']][i], theta[['sd']][i])*theta[['alpha']][i]))
+          function(i) dnorm(x, theta[['mu']][i],
+             theta[['sd']][i])*theta[['alpha']][i]))
   sum.of.components <- rowSums(product)
   posterior <- product / sum.of.components
   
@@ -886,7 +1018,8 @@ enve.recplot2.findPeaks.__em_m <- function
   components <- ncol(posterior)
   n <- colSums(posterior)
   mu <- colSums(posterior * x) / n
-  sd <- sqrt( colSums(posterior * (matrix(rep(x,components), ncol=components) - mu)^2) / n )
+  sd <- sqrt( colSums(
+        posterior * (matrix(rep(x,components), ncol=components) - mu)^2) / n )
   alpha <- n/length(x)
   return(list(mu=mu, sd=sd, alpha=alpha))
 }
@@ -1002,7 +1135,10 @@ enve.recplot2.findPeaks.__mower <- function
 enve.recplot2.__whichClosestPeak <- function
    ### Internal ancilliary function (see `enve.recplot2.findPeaks`).
       (peak, peaks){
-   dist <- as.numeric(lapply(peaks, function(x) abs(log(x$param.hat[[ length(x$param.hat) ]]/peak$param.hat[[ length(peak$param.hat) ]] ))))
+   dist <- as.numeric(lapply(peaks,
+         function(x)
+           abs(log(x$param.hat[[ length(x$param.hat) ]] /
+             peak$param.hat[[ length(peak$param.hat) ]] ))))
    dist[ dist==0 ] <- Inf
    return(which.min(dist))
 }
